@@ -1,6 +1,4 @@
 /*
-   +----------------------------------------------------------------------+
-   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) The PHP Group                                          |
    +----------------------------------------------------------------------+
@@ -22,6 +20,7 @@
 #include "php_globals.h"
 #include "ext/standard/basic_functions.h"
 #include "ext/standard/file.h"
+#include "ext/standard/user_filters_arginfo.h"
 
 #define PHP_STREAM_BRIGADE_RES_NAME	"userfilter.bucket brigade"
 #define PHP_STREAM_BUCKET_RES_NAME "userfilter.bucket"
@@ -40,28 +39,26 @@ static int le_bucket;
 
 /* define the base filter class */
 
-PHP_FUNCTION(user_filter_nop)
+PHP_METHOD(php_user_filter, filter)
 {
+	zval *in, *out, *consumed;
+	zend_bool closing;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "rrzb", &in, &out, &consumed, &closing) == FAILURE) {
+		RETURN_THROWS();
+	}
 }
-ZEND_BEGIN_ARG_INFO(arginfo_php_user_filter_filter, 0)
-	ZEND_ARG_INFO(0, in)
-	ZEND_ARG_INFO(0, out)
-	ZEND_ARG_INFO(1, consumed)
-	ZEND_ARG_INFO(0, closing)
-ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO(arginfo_php_user_filter_onCreate, 0)
-ZEND_END_ARG_INFO()
+PHP_METHOD(php_user_filter, onCreate)
+{
+	ZEND_PARSE_PARAMETERS_NONE();
 
-ZEND_BEGIN_ARG_INFO(arginfo_php_user_filter_onClose, 0)
-ZEND_END_ARG_INFO()
+	RETURN_TRUE;
+}
 
-static const zend_function_entry user_filter_class_funcs[] = {
-	PHP_NAMED_FE(filter,	PHP_FN(user_filter_nop),		arginfo_php_user_filter_filter)
-	PHP_NAMED_FE(onCreate,	PHP_FN(user_filter_nop),		arginfo_php_user_filter_onCreate)
-	PHP_NAMED_FE(onClose,	PHP_FN(user_filter_nop),		arginfo_php_user_filter_onClose)
-	PHP_FE_END
-};
+PHP_METHOD(php_user_filter, onClose)
+{
+	ZEND_PARSE_PARAMETERS_NONE();
+}
 
 static zend_class_entry user_filter_class_entry;
 
@@ -78,7 +75,7 @@ PHP_MINIT_FUNCTION(user_filters)
 {
 	zend_class_entry *php_user_filter;
 	/* init the filter class ancestor */
-	INIT_CLASS_ENTRY(user_filter_class_entry, "php_user_filter", user_filter_class_funcs);
+	INIT_CLASS_ENTRY(user_filter_class_entry, "php_user_filter", class_php_user_filter_methods);
 	if ((php_user_filter = zend_register_internal_class(&user_filter_class_entry)) == NULL) {
 		return FAILURE;
 	}
@@ -164,7 +161,7 @@ php_stream_filter_status_t userfilter_filter(
 	zval func_name;
 	zval retval;
 	zval args[4];
-	zval zpropname;
+	zend_string *propname;
 	int call_result;
 
 	/* the userfilter object probably doesn't exist anymore */
@@ -172,10 +169,6 @@ php_stream_filter_status_t userfilter_filter(
 		return ret;
 	}
 
-	/* Make sure the stream is not closed while the filter callback executes. */
-	uint32_t orig_no_fclose = stream->flags & PHP_STREAM_FLAG_NO_FCLOSE;
-	stream->flags |= PHP_STREAM_FLAG_NO_FCLOSE;
-
 	if (!zend_hash_str_exists_ind(Z_OBJPROP_P(obj), "stream", sizeof("stream")-1)) {
 		zval tmp;
 
@@ -198,15 +191,15 @@ php_stream_filter_status_t userfilter_filter(
 	} else {
 		ZVAL_NULL(&args[2]);
 	}
+	ZVAL_MAKE_REF(&args[2]);
 
 	ZVAL_BOOL(&args[3], flags & PSFS_FLAG_FLUSH_CLOSE);
 
-	call_result = call_user_function_ex(NULL,
+	call_result = call_user_function(NULL,
 			obj,
 			&func_name,
 			&retval,
-			4, args,
-			0, NULL);
+			4, args);
 
 	zval_ptr_dtor(&func_name);
 
@@ -214,7 +207,7 @@ php_stream_filter_status_t userfilter_filter(
 		convert_to_long(&retval);
 		ret = (int)Z_LVAL(retval);
 	} else if (call_result == FAILURE) {
-		php_error_docref(NULL, E_WARNING, "failed to call filter function");
+		php_error_docref(NULL, E_WARNING, "Failed to call filter function");
 	}
 
 	if (bytes_consumed) {
@@ -243,18 +236,15 @@ php_stream_filter_status_t userfilter_filter(
 	/* filter resources are cleaned up by the stream destructor,
 	 * keeping a reference to the stream resource here would prevent it
 	 * from being destroyed properly */
-	ZVAL_STRINGL(&zpropname, "stream", sizeof("stream")-1);
-	Z_OBJ_HANDLER_P(obj, unset_property)(obj, &zpropname, NULL);
-	zval_ptr_dtor(&zpropname);
+	propname = zend_string_init("stream", sizeof("stream")-1, 0);
+	Z_OBJ_HANDLER_P(obj, unset_property)(Z_OBJ_P(obj), propname, NULL);
+	zend_string_release_ex(propname, 0);
 
 	zval_ptr_dtor(&args[3]);
 	zval_ptr_dtor(&args[2]);
 	zval_ptr_dtor(&args[1]);
 	zval_ptr_dtor(&args[0]);
 
-	stream->flags &= ~PHP_STREAM_FLAG_NO_FCLOSE;
-	stream->flags |= orig_no_fclose;
-
 	return ret;
 }
 
@@ -277,7 +267,7 @@ static php_stream_filter *user_filter_factory_create(const char *filtername,
 	/* some sanity checks */
 	if (persistent) {
 		php_error_docref(NULL, E_WARNING,
-				"cannot use a user-space filter with a persistent stream");
+				"Cannot use a user-space filter with a persistent stream");
 		return NULL;
 	}
 
@@ -311,18 +301,14 @@ static php_stream_filter *user_filter_factory_create(const char *filtername,
 			}
 			efree(wildcard);
 		}
-		if (fdat == NULL) {
-			php_error_docref(NULL, E_WARNING,
-					"Err, filter \"%s\" is not in the user-filter map, but somehow the user-filter-factory was invoked for it!?", filtername);
-			return NULL;
-		}
+		ZEND_ASSERT(fdat);
 	}
 
 	/* bind the classname to the actual class */
 	if (fdat->ce == NULL) {
 		if (NULL == (fdat->ce = zend_lookup_class(fdat->classname))) {
 			php_error_docref(NULL, E_WARNING,
-					"user-filter \"%s\" requires class \"%s\", but that class is not defined",
+					"User-filter \"%s\" requires class \"%s\", but that class is not defined",
 					filtername, ZSTR_VAL(fdat->classname));
 			return NULL;
 		}
@@ -358,6 +344,8 @@ static php_stream_filter *user_filter_factory_create(const char *filtername,
 			&retval,
 			0, NULL);
 
+	zval_ptr_dtor(&func_name);
+
 	if (Z_TYPE(retval) != IS_UNDEF) {
 		if (Z_TYPE(retval) == IS_FALSE) {
 			/* User reported filter creation error "return false;" */
@@ -375,7 +363,6 @@ static php_stream_filter *user_filter_factory_create(const char *filtername,
 		}
 		zval_ptr_dtor(&retval);
 	}
-	zval_ptr_dtor(&func_name);
 
 	/* set the filter property, this will be used during cleanup */
 	ZVAL_RES(&zfilter, zend_register_resource(filter, le_userfilters));
@@ -398,8 +385,7 @@ static void filter_item_dtor(zval *zv)
 	efree(fdat);
 }
 
-/* {{{ proto object stream_bucket_make_writeable(resource brigade)
-   Return a bucket object from the brigade for operating on */
+/* {{{ Return a bucket object from the brigade for operating on */
 PHP_FUNCTION(stream_bucket_make_writeable)
 {
 	zval *zbrigade, zbucket;
@@ -408,11 +394,11 @@ PHP_FUNCTION(stream_bucket_make_writeable)
 
 	ZEND_PARSE_PARAMETERS_START(1, 1)
 		Z_PARAM_RESOURCE(zbrigade)
-	ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+	ZEND_PARSE_PARAMETERS_END();
 
 	if ((brigade = (php_stream_bucket_brigade*)zend_fetch_resource(
 					Z_RES_P(zbrigade), PHP_STREAM_BRIGADE_RES_NAME, le_bucket_brigade)) == NULL) {
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	ZVAL_NULL(return_value);
@@ -440,20 +426,20 @@ static void php_stream_bucket_attach(int append, INTERNAL_FUNCTION_PARAMETERS)
 	ZEND_PARSE_PARAMETERS_START(2, 2)
 		Z_PARAM_RESOURCE(zbrigade)
 		Z_PARAM_OBJECT(zobject)
-	ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+	ZEND_PARSE_PARAMETERS_END();
 
 	if (NULL == (pzbucket = zend_hash_str_find_deref(Z_OBJPROP_P(zobject), "bucket", sizeof("bucket")-1))) {
-		php_error_docref(NULL, E_WARNING, "Object has no bucket property");
-		RETURN_FALSE;
+		zend_argument_value_error(2, "must be an object that has a \"bucket\" property");
+		RETURN_THROWS();
 	}
 
 	if ((brigade = (php_stream_bucket_brigade*)zend_fetch_resource(
 					Z_RES_P(zbrigade), PHP_STREAM_BRIGADE_RES_NAME, le_bucket_brigade)) == NULL) {
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	if ((bucket = (php_stream_bucket *)zend_fetch_resource_ex(pzbucket, PHP_STREAM_BUCKET_RES_NAME, le_bucket)) == NULL) {
-		RETURN_FALSE;
+		RETURN_THROWS();
 	}
 
 	if (NULL != (pzdata = zend_hash_str_find_deref(Z_OBJPROP_P(zobject), "data", sizeof("data")-1)) && Z_TYPE_P(pzdata) == IS_STRING) {
@@ -481,24 +467,21 @@ static void php_stream_bucket_attach(int append, INTERNAL_FUNCTION_PARAMETERS)
 }
 /* }}} */
 
-/* {{{ proto void stream_bucket_prepend(resource brigade, object bucket)
-   Prepend bucket to brigade */
+/* {{{ Prepend bucket to brigade */
 PHP_FUNCTION(stream_bucket_prepend)
 {
 	php_stream_bucket_attach(0, INTERNAL_FUNCTION_PARAM_PASSTHRU);
 }
 /* }}} */
 
-/* {{{ proto void stream_bucket_append(resource brigade, object bucket)
-   Append bucket to brigade */
+/* {{{ Append bucket to brigade */
 PHP_FUNCTION(stream_bucket_append)
 {
 	php_stream_bucket_attach(1, INTERNAL_FUNCTION_PARAM_PASSTHRU);
 }
 /* }}} */
 
-/* {{{ proto resource stream_bucket_new(resource stream, string buffer)
-   Create a new bucket for use on the current stream */
+/* {{{ Create a new bucket for use on the current stream */
 PHP_FUNCTION(stream_bucket_new)
 {
 	zval *zstream, zbucket;
@@ -511,20 +494,15 @@ PHP_FUNCTION(stream_bucket_new)
 	ZEND_PARSE_PARAMETERS_START(2, 2)
 		Z_PARAM_ZVAL(zstream)
 		Z_PARAM_STRING(buffer, buffer_len)
-	ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+	ZEND_PARSE_PARAMETERS_END();
 
 	php_stream_from_zval(stream, zstream);
-
 	pbuffer = pemalloc(buffer_len, php_stream_is_persistent(stream));
 
 	memcpy(pbuffer, buffer, buffer_len);
 
 	bucket = php_stream_bucket_new(stream, pbuffer, buffer_len, 1, php_stream_is_persistent(stream));
 
-	if (bucket == NULL) {
-		RETURN_FALSE;
-	}
-
 	ZVAL_RES(&zbucket, zend_register_resource(bucket, le_bucket));
 	object_init(return_value);
 	add_property_zval(return_value, "bucket", &zbucket);
@@ -535,16 +513,13 @@ PHP_FUNCTION(stream_bucket_new)
 }
 /* }}} */
 
-/* {{{ proto array stream_get_filters(void)
-   Returns a list of registered filters */
+/* {{{ Returns a list of registered filters */
 PHP_FUNCTION(stream_get_filters)
 {
 	zend_string *filter_name;
 	HashTable *filters_hash;
 
-	if (zend_parse_parameters_none() == FAILURE) {
-		return;
-	}
+	ZEND_PARSE_PARAMETERS_NONE();
 
 	array_init(return_value);
 
@@ -561,8 +536,7 @@ PHP_FUNCTION(stream_get_filters)
 }
 /* }}} */
 
-/* {{{ proto bool stream_filter_register(string filtername, string classname)
-   Registers a custom filter handler class */
+/* {{{ Registers a custom filter handler class */
 PHP_FUNCTION(stream_filter_register)
 {
 	zend_string *filtername, *classname;
@@ -571,18 +545,16 @@ PHP_FUNCTION(stream_filter_register)
 	ZEND_PARSE_PARAMETERS_START(2, 2)
 		Z_PARAM_STR(filtername)
 		Z_PARAM_STR(classname)
-	ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
-
-	RETVAL_FALSE;
+	ZEND_PARSE_PARAMETERS_END();
 
 	if (!ZSTR_LEN(filtername)) {
-		php_error_docref(NULL, E_WARNING, "Filter name cannot be empty");
-		return;
+		zend_argument_value_error(1, "must be a non-empty string");
+		RETURN_THROWS();
 	}
 
 	if (!ZSTR_LEN(classname)) {
-		php_error_docref(NULL, E_WARNING, "Class name cannot be empty");
-		return;
+		zend_argument_value_error(2, "must be a non-empty string");
+		RETURN_THROWS();
 	}
 
 	if (!BG(user_filter_map)) {
@@ -599,6 +571,7 @@ PHP_FUNCTION(stream_filter_register)
 	} else {
 		zend_string_release_ex(classname, 0);
 		efree(fdat);
+		RETVAL_FALSE;
 	}
 }
 /* }}} */
